@@ -358,9 +358,16 @@ size_t Archive::ReadHeader15()
               byte *D=&hd->SubData[8];
               RecoverySize=D[0]+((uint)D[1]<<8)+((uint)D[2]<<16)+((uint)D[3]<<24);
               RecoverySize*=512; // Sectors to size.
-              RecoveryPercent=ToPercent(RecoverySize,Tell());
+              int64 CurPos=Tell();
+              RecoveryPercent=ToPercent(RecoverySize,CurPos);
+              // Round fractional percent exceeding .5 to upper value.
+              if (ToPercent(RecoverySize+CurPos/200,CurPos)>RecoveryPercent)
+                RecoveryPercent++;
             }
           }
+
+          if (hd->CmpName(SUBHEAD_TYPE_CMT))
+            MainComment=true;
         }
         if ((hd->Flags & LHD_SALT)!=0)
           Raw.GetB(hd->Salt,SIZE_SALT30);
@@ -737,6 +744,24 @@ size_t Archive::ReadHeader50()
 
         if (ExtraSize!=0)
           ProcessExtra50(&Raw,(size_t)ExtraSize,&MainHead);
+
+#ifdef USE_QOPEN
+        if (MainHead.Locator && MainHead.QOpenOffset>0 && Cmd->QOpenMode!=QOPEN_NONE)
+        {
+          // We seek to QO block in the end of archive when processing
+          // QOpen.Load, so we need to preserve current block positions
+          // to not break normal archive processing by calling function.
+          int64 SaveCurBlockPos=CurBlockPos,SaveNextBlockPos=NextBlockPos;
+          HEADER_TYPE SaveCurHeaderType=CurHeaderType;
+          
+          QOpen.Init(this,false);
+          QOpen.Load(MainHead.QOpenOffset);
+
+          CurBlockPos=SaveCurBlockPos;
+          NextBlockPos=SaveNextBlockPos;
+          CurHeaderType=SaveCurHeaderType;
+        }
+#endif
       }
       break;
     case HEAD_FILE:
@@ -816,6 +841,9 @@ size_t Archive::ReadHeader50()
           ConvertFileHeader(hd);
         }
 
+        if (!FileBlock && hd->CmpName(SUBHEAD_TYPE_CMT))
+          MainComment=true;
+
           
         if (BadCRC)
         {
@@ -1275,9 +1303,10 @@ void Archive::ConvertFileHeader(FileHeader *hd)
     // This code must be performed only after other path separator checks,
     // because it produces backslashes illegal for some of checks above.
     // Backslash is allowed in file names in Unix, but not in Windows.
+    // Still, RAR 4.x uses backslashes as path separator even in Unix.
     // Forward slash is not allowed in both systems. In RAR 5.0 we use
-    // the backslash as universal path separator.
-    if (*s=='/' || *s=='\\' && hd->HSType!=HSYS_UNIX && Format!=RARFMT50)
+    // the forward slash as universal path separator.
+    if (*s=='/' || *s=='\\' && Format!=RARFMT50)
       *s=CPATHDIVIDER;
   }
 }
